@@ -60,6 +60,7 @@ typedef struct {
 struct TotemPlParserPrivate
 {
 	GList *ignore_schemes;
+	GList *ignore_mimetypes;
 	guint recurse_level;
 	gboolean fallback;
 };
@@ -67,6 +68,8 @@ struct TotemPlParserPrivate
 /* Signals */
 enum {
 	ENTRY,
+	PLAYLIST_START,
+	PLAYLIST_END,
 	LAST_SIGNAL
 };
 
@@ -98,6 +101,22 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      NULL, NULL,
 			      totemplparser_marshal_VOID__STRING_STRING_STRING,
 			      G_TYPE_NONE, 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	totem_pl_parser_table_signals[PLAYLIST_START] =
+		g_signal_new ("playlist-start",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_start),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
+	totem_pl_parser_table_signals[PLAYLIST_START] =
+		g_signal_new ("playlist-end",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_end),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
 }
 
 GQuark
@@ -167,8 +186,11 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 		return NULL;
 	}
 
-	/* Return the file. */
-	*data = g_realloc (buffer, total_bytes_read);
+	/* Return the file null-terminated. */
+	buffer = g_realloc (buffer, total_bytes_read + 1);
+	buffer[total_bytes_read] = '\0';
+	*data = buffer;
+
 	mimetype = gnome_vfs_get_mime_type_for_data (*data, total_bytes_read);
 
 	return mimetype;
@@ -293,8 +315,9 @@ totem_pl_parser_relative (const char *url, const char *output)
 
 static gboolean
 totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
-                           TotemPlParserIterFunc func, 
-                           const char *output, gpointer user_data, GError **error)
+			   TotemPlParserIterFunc func, 
+			   const char *output, const char *title,
+			   gpointer user_data, GError **error)
 {
 	GnomeVFSHandle *handle;
 	GnomeVFSResult res;
@@ -329,6 +352,17 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 	if (success == FALSE)
 		return FALSE;
 
+	if (title != NULL) {
+		buf = g_strdup_printf ("X-GNOME-Title=%s\n", title);
+		success = write_string (handle, buf, error);
+		g_free (buf);
+		if (success == FALSE)
+		{
+			gnome_vfs_close (handle);
+			return FALSE;
+		}
+	}
+
 	buf = g_strdup_printf ("NumberOfEntries=%d\n", num_entries);
 	success = write_string (handle, buf, error);
 	g_free (buf);
@@ -493,15 +527,17 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 }
 
 gboolean
-totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
-		TotemPlParserIterFunc func, const char *output,
-		TotemPlParserType type, gpointer user_data, GError **error)
+totem_pl_parser_write_with_title (TotemPlParser *parser, GtkTreeModel *model,
+				  TotemPlParserIterFunc func,
+				  const char *output, const char *title,
+				  TotemPlParserType type,
+				  gpointer user_data, GError **error)
 {
 	switch (type)
 	{
 	case TOTEM_PL_PARSER_PLS:
 		return totem_pl_parser_write_pls (parser, model, func,
-				output, user_data, error);
+				output, title, user_data, error);
 	case TOTEM_PL_PARSER_M3U:
 	case TOTEM_PL_PARSER_M3U_DOS:
 		return totem_pl_parser_write_m3u (parser, model, func,
@@ -514,6 +550,17 @@ totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 	return FALSE;
 }
 
+gboolean
+totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
+		       TotemPlParserIterFunc func,
+		       const char *output, TotemPlParserType type,
+		       gpointer user_data,
+		       GError **error)
+{
+	return totem_pl_parser_write_with_title (parser, model, func, output,
+			NULL, type, user_data, error);
+}
+
 static int
 read_ini_line_int (char **lines, const char *key)
 {
@@ -590,6 +637,11 @@ totem_pl_parser_finalize (GObject *object)
 	g_return_if_fail (parser->priv != NULL);
 
 	g_list_foreach (parser->priv->ignore_schemes, (GFunc) g_free, NULL);
+	g_list_free (parser->priv->ignore_schemes);
+
+	g_list_foreach (parser->priv->ignore_mimetypes, (GFunc) g_free, NULL);
+	g_list_free (parser->priv->ignore_mimetypes);
+
 	g_free (parser->priv);
 	parser->priv = NULL;
 
@@ -634,9 +686,6 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	/* figure out whether we're a unix or dos RAM file */
 	if (strstr(contents,"\x0d") == NULL)
 		split_char = "\n";
@@ -718,9 +767,6 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	/* is TRUE if there's an EXTINF on the previous line */
 	extinfo = FALSE;
 
@@ -794,16 +840,19 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 		gpointer data)
 {
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	char *contents, **lines, *ref;
+	char *contents, **lines, *ref, *split_char;
 	int size;
 
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
+	if (strstr(contents,"\x0d") == NULL) {
+		split_char = "\n";
+	} else {
+		split_char = "\x0d\n";
+	}
 
-	lines = g_strsplit (contents, "\n", 0);
+	lines = g_strsplit (contents, split_char, 0);
 	g_free (contents);
 
 	ref = read_ini_line_string (lines, "Ref1", FALSE);
@@ -832,7 +881,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, **lines;
 	int size, i, num_entries;
-	char *split_char;
+	char *split_char, *playlist_title;
 	gboolean dos_mode = FALSE;
 
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
@@ -844,12 +893,8 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	/* figure out whether we're a unix pls or dos pls */
-	if (strstr(contents,"\x0d") == NULL)
-	{
+	if (strstr(contents,"\x0d") == NULL) {
 		split_char = "\n";
 	} else {
 		split_char = "\x0d\n";
@@ -860,6 +905,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 
 	/* [playlist] */
 	i = 0;
+	playlist_title = NULL;
 
 	/* Ignore empty lines */
 	while (lines[i] != NULL && strcmp (lines[i], "") == 0)
@@ -870,6 +916,15 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 				(gsize)strlen ("[playlist]")) != 0)
 		goto bail;
 
+	playlist_title = read_ini_line_string (lines,
+			"X-GNOME-Title", dos_mode);
+
+	if (playlist_title != NULL) {
+		g_signal_emit (G_OBJECT (parser),
+				totem_pl_parser_table_signals[PLAYLIST_START],
+				0, playlist_title);
+	}
+
 	/* numberofentries=? */
 	num_entries = read_ini_line_int (lines, "numberofentries");
 	if (num_entries == -1)
@@ -928,7 +983,14 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 		g_free (genre);
 	}
 
+	if (playlist_title != NULL) {
+		g_signal_emit (G_OBJECT (parser),
+				totem_pl_parser_table_signals[PLAYLIST_END],
+				0, playlist_title);
+	}
+
 bail:
+	g_free (playlist_title);
 	g_strfreev (lines);
 
 	return retval;
@@ -941,6 +1003,7 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 	xmlNodePtr node;
 	guchar *title, *url;
 	gboolean retval = FALSE;
+	char *fullpath = NULL;
 
 	title = NULL;
 	url = NULL;
@@ -967,22 +1030,19 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		return FALSE;
 	}
 
-	if (strstr ((char *)url, "://") != NULL || url[0] == '/') {
-		totem_pl_parser_add_one_url (parser, (char *)url,
-				(char *)title ? (char *)title : pl_title);
-		retval = TRUE;
-	} else {
-		char *fullpath;
-
+	if (strstr ((char *)url, "://") == NULL && url[0] != '/') {
 		fullpath = g_strdup_printf ("%s/%s", base, url);
-		/* .asx files can contain references to other .asx files */
-		if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS)
-			totem_pl_parser_add_one_url (parser, fullpath,
-					(char *)title ? (char *)title : pl_title);
-
-		g_free (fullpath);
+	} else {
+		fullpath = g_strdup ((char *)url);
 	}
 
+	/* .asx files can contain references to other .asx files */
+	if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		totem_pl_parser_add_one_url (parser, fullpath,
+				(char *)title ? (char *)title : pl_title);
+	}
+
+	g_free (fullpath);
 	g_free (title);
 	g_free (url);
 
@@ -1028,23 +1088,6 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 	return retval;
 }
 
-static TotemPlParserResult
-totem_pl_parser_add_block (TotemPlParser *parser, const char *url, gpointer data)
-{
-	MediaType type;
-	char *media_url;
-
-	type = totem_cd_detect_type_with_url (url, &media_url, NULL);
-	if (type == MEDIA_TYPE_DATA || media_url == NULL)
-		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	else if (type == MEDIA_TYPE_ERROR)
-		return TOTEM_PL_PARSER_RESULT_ERROR;
-
-	totem_pl_parser_add_one_url (parser, media_url, NULL);
-	g_free (media_url);
-	return TOTEM_PL_PARSER_RESULT_SUCCESS;
-}
-
 static TotemPlParserResult
 totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 {
@@ -1057,9 +1100,6 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	doc = xmlParseMemory (contents, size);
 	if (doc == NULL)
 		doc = xmlRecoverMemory (contents, size);
@@ -1188,9 +1228,6 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	doc = xmlParseMemory (contents, size);
 	if (doc == NULL)
 		doc = xmlRecoverMemory (contents, size);
@@ -1242,9 +1279,6 @@ totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer da
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	contents = g_realloc (contents, size + 1);
-	contents[size] = '\0';
-
 	lines = g_strsplit (contents, "\n", 0);
 	g_free (contents);
 
@@ -1322,6 +1356,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	while (l != NULL) {
 		char *name, *fullpath;
 		GnomeVFSFileInfo *info = l->data;
+		TotemPlParserResult ret;
 
 		if (info->name != NULL && (strcmp (info->name, ".") == 0
 					|| strcmp (info->name, "..") == 0)) {
@@ -1333,7 +1368,8 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 		fullpath = g_strconcat (url, "/", name, NULL);
 		g_free (name);
 
-		if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS)
+		ret = totem_pl_parser_parse_internal (parser, fullpath);
+		if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && ret != TOTEM_PL_PARSER_RESULT_IGNORED)
 			totem_pl_parser_add_one_url (parser, fullpath, NULL);
 
 		l = l->next;
@@ -1357,7 +1393,13 @@ static PlaylistTypes special_types[] = {
 	{ "x-directory/normal", totem_pl_parser_add_directory },
 	{ "video/x-ms-wvx", totem_pl_parser_add_asx },
 	{ "audio/x-ms-wax", totem_pl_parser_add_asx },
-	{ "x-special/device-block", totem_pl_parser_add_block },
+};
+
+static PlaylistTypes ignore_types[] = {
+	{ "image/*", NULL },
+	{ "text/plain", NULL },
+	{ "application/x-rar", NULL },
+	{ "application/zip", NULL },
 };
 
 /* These ones are "dual" types, might be a video, might be a parser */
@@ -1389,6 +1431,26 @@ totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, const char *url)
 	return FALSE;
 }
 
+static gboolean
+totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
+				     const char *mimetype)
+{
+	GList *l;
+
+	if (parser->priv->ignore_mimetypes == NULL)
+		return FALSE;
+
+	for (l = parser->priv->ignore_mimetypes; l != NULL; l = l->next)
+	{
+		const char *item = l->data;
+		if (strcmp (mimetype, item) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
+
+}
+
 static gboolean
 totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
 {
@@ -1427,6 +1489,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	guint i;
 	gpointer data = NULL;
 	gboolean ret = FALSE;
+	char *super;
 
 	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -1438,6 +1501,30 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	if (mimetype == NULL)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
+	if (totem_pl_parser_mimetype_is_ignored (parser, mimetype) != FALSE) {
+		g_free (data);
+		return TOTEM_PL_PARSER_RESULT_IGNORED;
+	}
+
+	super = gnome_vfs_get_supertype_from_mime_type (mimetype);
+	for (i = 0; i < G_N_ELEMENTS (ignore_types); i++) {
+		if (gnome_vfs_mime_type_is_supertype (ignore_types[i].mimetype) != FALSE) {
+			if (strcmp (super, ignore_types[i].mimetype) == 0) {
+				g_free (data);
+				g_free (super);
+				return TOTEM_PL_PARSER_RESULT_IGNORED;
+			}
+		} else {
+			GnomeVFSMimeEquivalence eq;
+
+			eq = gnome_vfs_mime_type_get_equivalence (mimetype, ignore_types[i].mimetype);
+			if (eq == GNOME_VFS_MIME_PARENT || eq == GNOME_VFS_MIME_IDENTICAL) {
+				g_free (data);
+				return TOTEM_PL_PARSER_RESULT_IGNORED;
+			}
+		}
+	}
+
 	parser->priv->recurse_level++;
 
 	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
@@ -1479,11 +1566,11 @@ totem_pl_parser_parse (TotemPlParser *parser, const char *url,
 	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	g_return_val_if_fail (url != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
 
-	if (totem_pl_parser_ignore (parser, url) != FALSE)
+	if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
 	g_return_val_if_fail (strstr (url, "://") != NULL,
-			TOTEM_PL_PARSER_RESULT_UNHANDLED);
+			TOTEM_PL_PARSER_RESULT_IGNORED);
 
 	parser->priv->recurse_level = 0;
 	parser->priv->fallback = fallback;
@@ -1500,3 +1587,13 @@ totem_pl_parser_add_ignored_scheme (TotemPlParser *parser,
 		(parser->priv->ignore_schemes, g_strdup (scheme));
 }
 
+void
+totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
+		const char *mimetype)
+{
+	g_return_if_fail (TOTEM_IS_PL_PARSER (parser));
+
+	parser->priv->ignore_mimetypes = g_list_prepend
+		(parser->priv->ignore_mimetypes, g_strdup (mimetype));
+}
+
