@@ -27,17 +27,15 @@
 
 #include "config.h"
 
-#ifndef HAVE_GTK_ONLY
-#include <gnome.h>
-#else
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#endif
 
 #include <popt.h>
 
+#include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
+#include <libgnomevfs/gnome-vfs-init.h>
 #include "bacon-video-widget.h"
 
 /* #define THUMB_DEBUG */
@@ -342,6 +340,7 @@ int main (int argc, char *argv[])
 	gtk_init (&argc, &argv);
 #endif
 	bacon_video_widget_init_backend (&argc, &argv);
+	gnome_vfs_init ();
 
 	if (!strcmp (argv[1], "-s")) {
 		input = argv[3];
