@@ -53,10 +53,12 @@
 #include "totem-time-label.h"
 #include "totem-session.h"
 #include "totem-screenshot.h"
+#include "totem-sidebar.h"
 #include "totem-menu.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
+#include "totem-volume.h"
 #include "video-utils.h"
 
 #include "totem.h"
@@ -160,26 +162,34 @@ totem_action_save_size (Totem *totem)
 void
 totem_action_exit (Totem *totem)
 {
+	GdkDisplay *display = NULL;
+
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
 
 	if (totem == NULL)
 		exit (0);
 
+	if (totem->playlist != NULL)
+		gtk_widget_hide (GTK_WIDGET (totem->playlist));
+
+	if (totem->win != NULL) {
+		gtk_widget_hide (totem->win);
+		display = gtk_widget_get_display (totem->win);
+	}
+
 #ifndef HAVE_GTK_ONLY
 	totem_gromit_clear (TRUE);
 #endif /* !HAVE_GTK_ONLY */
+
+	if (display != NULL)
+		gdk_display_sync (display);
+
 	bacon_message_connection_free (totem->conn);
 	totem_action_save_size (totem);
 
 	totem_action_fullscreen (totem, FALSE);
 
-	if (totem->playlist)
-		gtk_widget_hide (GTK_WIDGET (totem->playlist));
-
-	if (totem->win)
-		gtk_widget_hide (totem->win);
-
 	totem_sublang_exit (totem);
 	totem_named_icons_dispose (totem);
 
@@ -210,18 +220,6 @@ totem_action_menu_popup (Totem *totem, guint button)
 			button, gtk_get_current_event_time ());
 }
 
-static void
-action_toggle_playlist (Totem *totem)
-{
-	GtkWidget *toggle;
-	gboolean state;
-
-	toggle = glade_xml_get_widget (totem->xml, "tmw_playlist_button");
-
-	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), !state);
-}
-
 static gboolean
 main_window_destroy_cb (GtkWidget *widget, GdkEvent *event, Totem *totem)
 {
@@ -395,13 +393,74 @@ totem_action_load_media (Totem *totem, MediaType type)
 	return TRUE;
 }
 
+static gboolean
+totem_action_load_media_device (Totem *totem, const char *device)
+{
+	MediaType type;
+	GError *error = NULL;
+	char *device_path, *url;
+	gboolean retval;
+
+	if (g_str_has_prefix (device, "file://") != FALSE)
+		device_path = g_filename_from_uri (device, NULL, NULL);
+	else
+		device_path = g_strdup (device);
+
+	type = totem_cd_detect_type_with_url (device_path, &url, &error);
+
+	switch (type) {
+		case MEDIA_TYPE_ERROR:
+			totem_action_error ("Failed to play Audio/Video Disc",
+					    error ? error->message : "Reason unknown",
+					    totem);
+			retval = FALSE;
+			break;
+		case MEDIA_TYPE_DATA:
+			/* Set default location to the mountpoint of
+			 * this device */
+			{
+				char *s;
+				s = totem_action_open_dialog (totem, url, FALSE);
+				g_free (s);
+			}
+			retval = TRUE;
+			break;
+		case MEDIA_TYPE_DVD:
+		case MEDIA_TYPE_VCD:
+		case MEDIA_TYPE_CDDA:
+			bacon_video_widget_set_media_device
+				(BACON_VIDEO_WIDGET (totem->bvw), device_path);
+			totem_action_load_media (totem, type);
+			retval = TRUE;
+			break;
+		default:
+			g_assert_not_reached ();
+	}
+
+	g_free (url);
+	g_free (device_path);
+
+	return retval;
+}
+
+void
+totem_action_play_media_device (Totem *totem, const char *device)
+{
+	char *mrl;
+
+	if (totem_action_load_media_device (totem, device) != FALSE) {
+		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		totem_action_set_mrl_and_play (totem, mrl);
+		g_free (mrl);
+	}
+}
+
 void
 totem_action_play_media (Totem *totem, MediaType type)
 {
 	char *mrl;
 
-	if (totem_action_load_media (totem, type) != FALSE)
-	{
+	if (totem_action_load_media (totem, type) != FALSE) {
 		mrl = totem_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_and_play (totem, mrl);
 		g_free (mrl);
@@ -487,12 +546,19 @@ totem_action_fullscreen_toggle (Totem *totem)
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
 			show_controls (totem, TRUE);
 		}
+
+		if (totem_sidebar_is_visible (totem) != FALSE)
+			gtk_widget_show (totem->sidebar);
+		else
+			gtk_widget_hide (totem->sidebar);
 	} else {
 		totem_action_save_size (totem);
 		update_fullscreen_size (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
 		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
 		gtk_window_fullscreen (GTK_WINDOW(totem->win));
+		gtk_widget_hide (totem->sidebar);
+
 		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
 			totem_scrsaver_disable (totem->scr);
 
@@ -514,6 +580,7 @@ static char *
 totem_get_nice_name_for_stream (Totem *totem)
 {
 	char *title, *artist, *retval;
+	int tracknum;
 	GValue value = { 0, };
 
 	bacon_video_widget_get_metadata (totem->bvw, BVW_INFO_TITLE, &value);
@@ -528,12 +595,18 @@ totem_get_nice_name_for_stream (Totem *totem)
 	g_value_unset (&value);
 
 	if (artist == NULL)
-	{
-		g_free (title);
-		return NULL;
-	}
+		return title;
 
-	retval = g_strdup_printf ("%s - %s", artist, title);
+	bacon_video_widget_get_metadata (totem->bvw, BVW_INFO_TRACK_NUMBER,
+			&value);
+	tracknum = g_value_get_int (&value);
+
+	if (tracknum != 0) {
+		retval = g_strdup_printf ("%02d. %s - %s",
+				tracknum, artist, title);
+	} else {
+		retval = g_strdup_printf ("%s - %s", artist, title);
+	}
 	g_free (artist);
 	g_free (title);
 
@@ -573,15 +646,11 @@ update_mrl_label (Totem *totem, const char *name)
 
 		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
 		gtk_label_set_markup (GTK_LABEL (widget), text);
-		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
-		gtk_label_set_markup (GTK_LABEL (widget), text);
 
 		g_free (text);
 
 		/* Title */
-		text = g_strdup_printf (_("%s - Totem Movie Player"), name);
-		gtk_window_set_title (GTK_WINDOW (totem->win), text);
-		g_free (text);
+		gtk_window_set_title (GTK_WINDOW (totem->win), name);
 	} else {
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
 				(totem->statusbar), 0, 0);
@@ -596,8 +665,6 @@ update_mrl_label (Totem *totem, const char *name)
 			 _("No File"));
 		widget = glade_xml_get_widget (totem->xml, "tcw_title_label");
 		gtk_label_set_markup (GTK_LABEL (widget), text);
-		widget = glade_xml_get_widget (totem->xml, "tmw_title_label");
-		gtk_label_set_markup (GTK_LABEL (widget), text);
 
 		g_free (text);
 
@@ -628,8 +695,6 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 	{
 		retval = FALSE;
 
-		gtk_window_set_title (GTK_WINDOW (totem->win), _("Totem"));
-
 		/* Play/Pause */
 		gtk_widget_set_sensitive (totem->pp_button, FALSE);
 		widget = glade_xml_get_widget (totem->xml, "tmw_play_menu_item");
@@ -642,11 +707,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		update_seekable (totem, FALSE);
 
 		/* Volume */
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_hbox");
-		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_up_menu_item");
-		gtk_widget_set_sensitive (widget, FALSE);
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_down_menu_item");
+		widget = glade_xml_get_widget (totem->xml, "tcw_volume_button");
 		gtk_widget_set_sensitive (widget, FALSE);
 		widget = glade_xml_get_widget (totem->xml_popup, "trcm_volume_up");
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -708,11 +769,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		/* Volume */
 		caps = bacon_video_widget_can_set_volume (totem->bvw);
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_hbox");
-		gtk_widget_set_sensitive (widget, caps);
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_up_menu_item");
-		gtk_widget_set_sensitive (widget, caps);
-		widget = glade_xml_get_widget (totem->xml, "tmw_volume_down_menu_item");
+		widget = glade_xml_get_widget (totem->xml, "tcw_volume_button");
 		gtk_widget_set_sensitive (widget, caps);
 		widget = glade_xml_get_widget (totem->xml, "tcw_volume_hbox");
 		gtk_widget_set_sensitive (widget, caps);
@@ -964,7 +1021,7 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 	GList *list, *p, *file_list;
 	gboolean cleared = FALSE;
 
-	list = gnome_vfs_uri_list_parse (data->data);
+	list = gnome_vfs_uri_list_parse ((const char *)data->data);
 
 	if (list == NULL)
 		return FALSE;
@@ -981,11 +1038,14 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 	}
 
 	gnome_vfs_uri_list_free (list);
-	file_list = g_list_reverse (file_list);
-
 	if (file_list == NULL)
 		return FALSE;
 
+	if (drop_type != 1)
+		file_list = g_list_sort (file_list, (GCompareFunc) strcmp);
+	else
+		file_list = g_list_reverse (file_list);
+
 	for (p = file_list; p != NULL; p = p->next)
 	{
 		char *filename, *title;
@@ -1128,18 +1188,6 @@ on_next_button_clicked (GtkButton *button, Totem *totem)
 	TOTEM_PROFILE (totem_action_next (totem));
 }
 
-static void
-on_playlist_button_toggled (GtkToggleButton *button, Totem *totem)
-{
-	gboolean state;
-
-	state = gtk_toggle_button_get_active (button);
-	if (state != FALSE)
-		gtk_widget_show (GTK_WIDGET (totem->playlist));
-	else
-		gtk_widget_hide (GTK_WIDGET (totem->playlist));
-}
-
 static void
 on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
@@ -1314,6 +1362,9 @@ update_current_time (BaconVideoWidget *bvw,
 		totem_time_label_set_time
 			(TOTEM_TIME_LABEL (totem->tcw_time_label),
 			 current_time, stream_length);
+		bacon_video_widget_properties_from_time
+			(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
+			 stream_length);
 	}
 }
 
@@ -1342,8 +1393,8 @@ update_volume_sliders (Totem *totem)
 				totem->prev_volume != -1 && volume != -1))
 	{
 		totem->volume_first_time = 0;
-		gtk_adjustment_set_value (totem->voladj,
-				(float) volume);
+		totem_volume_button_set_value (
+			TOTEM_VOLUME_BUTTON (totem->volume), (float) volume);
 		gtk_adjustment_set_value (totem->fs_voladj,
 				(float) volume);
 	}
@@ -1423,17 +1474,16 @@ vol_cb (GtkWidget *widget, Totem *totem)
 				(totem->bvw, (gint) totem->fs_voladj->value);
 
 			/* Update the fullscreen volume adjustment */
-			gtk_adjustment_set_value (totem->voladj, 
-					gtk_adjustment_get_value
-					(totem->fs_voladj));
+			totem_volume_button_set_value (
+				TOTEM_VOLUME_BUTTON (totem->volume),
+				gtk_adjustment_get_value (totem->fs_voladj));
 		} else {
-			bacon_video_widget_set_volume
-				(totem->bvw, (gint) totem->voladj->value);
-			/* Update the volume adjustment */
-			gtk_adjustment_set_value (totem->fs_voladj, 
-					gtk_adjustment_get_value
-					(totem->voladj));
+			int value = totem_volume_button_get_value (
+				TOTEM_VOLUME_BUTTON (totem->volume));
 
+			bacon_video_widget_set_volume (totem->bvw, value);
+			/* Update the volume adjustment */
+			gtk_adjustment_set_value (totem->fs_voladj, value);
 		}
 
 		totem->vol_lock = FALSE;
@@ -1501,13 +1551,8 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				cleared = TRUE;
 			}
 
-			if (strcmp (data, "dvd:") == 0)
-			{
-				totem_action_load_media (totem, MEDIA_TYPE_DVD);
-			} else if (strcmp (data, "vcd:") == 0) {
-				totem_action_load_media (totem, MEDIA_TYPE_VCD);
-			} else if (strcmp (data, "cd:") == 0) {
-				totem_action_load_media (totem, MEDIA_TYPE_CDDA);
+			if (totem_is_block_device (filename) != FALSE) {
+				totem_action_load_media_device (totem, data);
 			} else if (strstr (filename, "cdda:/") != NULL) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
 			} else if (totem_playlist_add_mrl (totem->playlist,
@@ -1530,12 +1575,12 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 	return cleared;
 }
 
-static void
-on_open1_activate (GtkButton *button, Totem *totem)
+char *
+totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
 {
 	GtkWidget *fs;
 	int response;
-	static char *path = NULL;
+	char *path_new = NULL;
 
 	fs = gtk_file_chooser_dialog_new (_("Select Files"),
 			GTK_WINDOW (totem->win), GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -1556,7 +1601,6 @@ on_open1_activate (GtkButton *button, Totem *totem)
 	while (1)
 	{
 		GSList *filenames;
-		char *mrl;
 		gboolean playlist_modified;
 
 		response = gtk_dialog_run (GTK_DIALOG (fs));
@@ -1581,21 +1625,36 @@ on_open1_activate (GtkButton *button, Totem *totem)
 
 		/* Save the path */
 		if (filenames->data != NULL)
-		{
-			g_free (path);
-			path = g_path_get_dirname (filenames->data);
-		}
+			path_new = g_path_get_dirname (filenames->data);
 
 		g_slist_foreach (filenames, (GFunc) g_free, NULL);
 		g_slist_free (filenames);
 
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
+		if (play != FALSE) {
+			char *mrl;
+
+			mrl = totem_playlist_get_current_mrl (totem->playlist);
+			totem_action_set_mrl_and_play (totem, mrl);
+			g_free (mrl);
+		}
 		break;
 	}
 
 	gtk_widget_destroy (fs);
+	return path_new;
+}
+
+static void
+on_open1_activate (GtkButton *button, Totem *totem)
+{
+	static char *path = NULL;
+	char *new_path;
+
+	new_path = totem_action_open_dialog (totem, path, TRUE);
+	if (new_path != NULL) {
+		g_free (path);
+		path = new_path;
+	}
 }
 
 //FIXME move the URI to a separate widget
@@ -1728,12 +1787,6 @@ on_aspect_ratio_dvb_activate (GtkButton *button, Totem *totem)
 	totem_action_set_aspect_ratio (totem, BVW_RATIO_DVB);
 }
 
-static void
-on_show_playlist1_activate (GtkButton *button, Totem *totem)
-{
-	action_toggle_playlist (totem);
-}
-
 static void
 on_fs_exit1_activate (GtkButton *button, Totem *totem)
 {
@@ -1763,7 +1816,7 @@ on_shuffle_mode1_toggled (GtkCheckMenuItem *checkmenuitem, Totem *totem)
 static void
 on_always_on_top1_activate (GtkCheckMenuItem *checkmenuitem, Totem *totem)
 {
-	totem_gdk_window_set_always_on_top (GTK_WIDGET (totem->win)->window,
+	gtk_window_set_keep_above (GTK_WINDOW (totem->win),
 			gtk_check_menu_item_get_active (checkmenuitem));
 	gconf_client_set_bool (totem->gc,
 			GCONF_PREFIX"/window_on_top",
@@ -1776,6 +1829,9 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	GtkWidget *menubar, *controlbar, *statusbar, *item, *bvw_vbox, *widget;
 	int width = 0, height = 0;
 
+	if (totem->bvw == NULL)
+		return;
+
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
@@ -2264,17 +2320,6 @@ totem_button_pressed_remote_cb (TotemRemote *remote, TotemRemoteCommand cmd,
 }
 #endif /* HAVE_REMOTE */
 
-static int
-toggle_playlist_from_playlist (GtkWidget *playlist, int trash, Totem *totem)
-{
-	GtkWidget *button;
-
-	button = glade_xml_get_widget (totem->xml, "tmw_playlist_button");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
-
-	return TRUE;
-}
-
 static void
 playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 {
@@ -2474,8 +2519,6 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 
 	item = glade_xml_get_widget (totem->xml, "tcw_hbox");
 	gtk_widget_show_all (item);
-	item = glade_xml_get_widget (totem->xml, "tmw_title_label");
-	gtk_widget_realize (item);
 	gdk_flush ();
 
 	move_popups (totem);
@@ -2613,7 +2656,7 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 	case GDK_P:
 		/* Playlist keyboard shortcut? */
 		if (event->state & GDK_CONTROL_MASK)
-			action_toggle_playlist (totem);
+			totem_sidebar_toggle (totem);
 		else
 			totem_action_play_pause (totem);
 		break;
@@ -2951,9 +2994,6 @@ totem_callback_connect (Totem *totem)
 			"tmw_aspect_ratio_dvb_menu_item");
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_aspect_ratio_dvb_activate), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_show_playlist_menu_item");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_show_playlist1_activate), totem);
 	item = glade_xml_get_widget (totem->xml, "tmw_repeat_mode_menu_item");
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 			totem_playlist_get_repeat (totem->playlist));
@@ -3043,19 +3083,10 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (on_previous_button_clicked), totem);
 	item = glade_xml_get_widget (totem->xml, "tmw_next_button");
 	g_signal_connect (G_OBJECT (item), "clicked", 
-			G_CALLBACK (on_next_button_clicked), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_playlist_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_playlist_button_toggled), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_volume_mute_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_volume_mute_button), totem);
-	item = glade_xml_get_widget (totem->xml, "tmw_volume_max_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (on_volume_max_button), totem);
+			  G_CALLBACK (on_next_button_clicked), totem);
 
 	/* Drag'n'Drop */
-	item = glade_xml_get_widget (totem->xml, "tmw_playlist_button");
+	item = glade_xml_get_widget (totem->xml, "tmw_sidebar_button");
 	g_signal_connect (G_OBJECT (item), "drag_data_received",
 			G_CALLBACK (drop_playlist_cb), totem);
 	gtk_drag_dest_set (item, GTK_DEST_DEFAULT_ALL,
@@ -3162,13 +3193,6 @@ totem_callback_connect (Totem *totem)
 	g_signal_connect (G_OBJECT (totem->volume), "value-changed",
 			G_CALLBACK (vol_cb), totem);
 
-	/* Playlist Disappearance, woop woop */
-	g_signal_connect (G_OBJECT (totem->playlist), "response",
-			G_CALLBACK (toggle_playlist_from_playlist), totem);
-	g_signal_connect (G_OBJECT (totem->playlist), "delete-event",
-			G_CALLBACK (toggle_playlist_from_playlist),
-			totem);
-
 	/* Playlist */
 	g_signal_connect (G_OBJECT (totem->playlist),
 			"changed", G_CALLBACK (playlist_changed_cb),
@@ -3232,6 +3256,20 @@ totem_callback_connect (Totem *totem)
 	g_timeout_add (600, (GSourceFunc) gui_update_cb, totem);
 }
 
+static void
+playlist_widget_setup (Totem *totem)
+{
+	totem->playlist = TOTEM_PLAYLIST (totem_playlist_new ());
+
+	if (totem->playlist == NULL)
+		totem_action_exit (totem);
+
+	gtk_widget_show_all (GTK_WIDGET (totem->playlist));
+
+	g_signal_connect(totem->playlist, "active-name-changed",
+			G_CALLBACK (on_playlist_change_name), totem);
+}
+
 static void
 video_widget_create (Totem *totem) 
 {
@@ -3260,9 +3298,7 @@ video_widget_create (Totem *totem)
 
 	totem_preferences_tvout_setup (totem);
 	totem_preferences_visuals_setup (totem);
-
-	/* Let's set a name. Will make debugging easier */
-	gtk_widget_set_name (GTK_WIDGET(totem->bvw), "bvw");
+	totem_action_zoom (totem, ZOOM_RESET);
 
 	g_signal_connect (G_OBJECT (totem->bvw),
 			"motion-notify-event",
@@ -3344,28 +3380,6 @@ video_widget_create (Totem *totem)
 	update_volume_sliders (totem);
 }
 
-GtkWidget *
-totem_statusbar_create (void)
-{
-	GtkWidget *widget;
-
-	widget = totem_statusbar_new ();
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
-GtkWidget *
-totem_time_display_create (void)
-{
-	GtkWidget *widget;
-
-	widget = totem_time_label_new ();
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 static void
 totem_message_connection_receive_cb (const char *msg, Totem *totem)
 {
@@ -3389,6 +3403,17 @@ totem_message_connection_receive_cb (const char *msg, Totem *totem)
 	g_free (url);
 }
 
+GtkWidget *
+totem_volume_create (void)
+{
+	GtkWidget *widget;
+
+	widget = totem_volume_button_new (0, 100, -1);
+	gtk_widget_show (widget);
+
+	return widget;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -3471,20 +3496,16 @@ main (int argc, char **argv)
 
 	totem_named_icons_init (totem, FALSE);
 
-	/* The playlist */
-	totem->playlist = TOTEM_PLAYLIST (totem_playlist_new ());
-	if (totem->playlist == NULL)
-		totem_action_exit (totem);
-	g_signal_connect(totem->playlist, "active-name-changed",
-			 G_CALLBACK (on_playlist_change_name), totem);
+	/* The sidebar */
+	playlist_widget_setup (totem);
+	totem_sidebar_setup (totem);
 
 	/* The rest of the widgets */
 	totem->state = STATE_STOPPED;
 	totem->seek = glade_xml_get_widget (totem->xml, "tmw_seek_hscale");
 	totem->seekadj = gtk_range_get_adjustment (GTK_RANGE (totem->seek));
 	g_object_set_data (G_OBJECT (totem->seek), "fs", GINT_TO_POINTER (0));
-	totem->volume = glade_xml_get_widget (totem->xml, "tmw_volume_hscale");
-	totem->voladj = gtk_range_get_adjustment (GTK_RANGE (totem->volume));
+	totem->volume = glade_xml_get_widget (totem->xml, "tcw_volume_button");
 	g_object_set_data (G_OBJECT (totem->volume), "fs", GINT_TO_POINTER (0));
 	totem->exit_popup = glade_xml_get_widget
 		(totem->xml, "totem_exit_fullscreen_window");
@@ -3537,7 +3558,7 @@ main (int argc, char **argv)
 	{
 		totem_session_restore (totem, argv);
 	} else if (argc >= 1 && totem_action_open_files (totem, argv)) {
-			totem_action_play_pause (totem);
+		totem_action_play_pause (totem);
 	} else {
 		totem_action_set_mrl (totem, NULL);
 	}
